# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.non_linear_cell import NonLinearCell


class HigherOrderTetra(NonLinearCell):
    """
    HigherOrderTetra - A 3d cell that represents an arbitrary order
    higher_order tetrahedron
    
    Superclass: NonLinearCell
    
    HigherOrderTetra is a concrete implementation of Cell to
    represent a 3d tetrahedron using higher_order shape functions of user
    specified order.
    
    The number of points in a higher_order cell determines the order over
    which they are iterated relative to the parametric coordinate system
    of the cell. The first points that are reported are vertices. They
    appear in the same order in which they would appear in linear cells.
    Mid-edge points are reported next. They are reported in sequence. For
    two- and three-dimensional (_3d) cells, the following set of points to
    be reported are face points. Finally, 3d cells report points interior
    to their volume.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkHigherOrderTetra, obj, update, **traits)
    
    def _get_parametric_coords(self):
        return self._vtk_obj.GetParametricCoords()
    def _set_parametric_coords(self, arg):
        old_val = self._get_parametric_coords()
        self._wrap_call(self._vtk_obj.SetParametricCoords,
                        arg)
        self.trait_property_changed('parametric_coords', old_val, arg)
    parametric_coords = traits.Property(_get_parametric_coords, _set_parametric_coords, desc=\
        """
        Return a contiguous array of parametric coordinates of the points
        defining this cell. In other words, (px,py,pz, px,py,pz, etc..) 
        The coordinates are ordered consistent with the definition of the
        point ordering for the cell. This method returns a non-nullptr
        pointer when the cell is a primary type (i.e., is_primary_cell() is
        true). Note that 3d parametric coordinates are returned no matter
        what the topological dimension of the cell.
        """
    )

    def _get_order(self):
        return self._vtk_obj.GetOrder()
    order = traits.Property(_get_order, desc=\
        """
        
        """
    )

    def barycentric_index(self, *args):
        """
        V.barycentric_index(int, [int, ...], int)
        C++: static void BarycentricIndex(IdType index,
            IdType *bindex, IdType order)"""
        ret = self._wrap_call(self._vtk_obj.BarycentricIndex, *args)
        return ret

    def compute_order(self, *args):
        """
        V.compute_order() -> int
        C++: IdType ComputeOrder()
        V.compute_order(int) -> int
        C++: static IdType ComputeOrder(const IdType nPoints)"""
        ret = self._wrap_call(self._vtk_obj.ComputeOrder, *args)
        return ret

    def index(self, *args):
        """
        V.index((int, ...), int) -> int
        C++: static IdType Index(const IdType *bindex,
            IdType order)"""
        ret = self._wrap_call(self._vtk_obj.Index, *args)
        return ret

    def to_barycentric_index(self, *args):
        """
        V.to_barycentric_index(int, [int, ...])
        C++: void ToBarycentricIndex(IdType index, IdType *bindex)"""
        ret = self._wrap_call(self._vtk_obj.ToBarycentricIndex, *args)
        return ret

    def to_index(self, *args):
        """
        V.to_index((int, ...)) -> int
        C++: IdType ToIndex(const IdType *bindex)"""
        ret = self._wrap_call(self._vtk_obj.ToIndex, *args)
        return ret

    def get_edge_cell(self):
        """
        V.get_edge_cell() -> HigherOrderCurve
        C++: virtual HigherOrderCurve *getEdgeCell()"""
        ret = wrap_vtk(self._vtk_obj.getEdgeCell())
        return ret
        

    def get_face_cell(self):
        """
        V.get_face_cell() -> HigherOrderTriangle
        C++: virtual HigherOrderTriangle *getFaceCell()"""
        ret = wrap_vtk(self._vtk_obj.getFaceCell())
        return ret
        

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(HigherOrderTetra, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit HigherOrderTetra properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], []),
            title='Edit HigherOrderTetra properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit HigherOrderTetra properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

